package gaf3.demo.services.crud

import gaf3.core.data.ErrorResult
import gaf3.core.data.PagedData
import gaf3.core.exception.BusinessError
import gaf3.demo.services.crud.service.UserService
import gaf3.demo.services.crud.domain.CreateForm
import gaf3.demo.services.crud.domain.UpdateForm
import gaf3.demo.services.crud.entity.GafUserInfo
import org.springframework.web.bind.annotation.*
import javax.validation.Valid

@RestController
class UserServiceController(val service: UserService) {
//    companion object {
//        val log: Logger = LoggerFactory.getLogger(UserServiceController::class.java)
//    }

    @PostMapping("/gaf/users")
    fun create(@Valid @RequestBody data: CreateForm) : GafUserInfo {
        return service.create(data)
    }

    @GetMapping("/gaf/users")
    fun query(filter: GafUserInfo, skip: Int? = 0, limit: Int? = 100, exact: Boolean?) : PagedData<GafUserInfo> {
        return service.find(filter, skip?:0, limit?:100, exact ?: true)
    }

    @GetMapping("/gaf/users/{userId}")
    fun fetch(@PathVariable userId: String) : GafUserInfo {
        return service.fetch(userId) ?: throw BusinessError(BusinessError.ERR_DATA_NOTEXIST)
    }

    @DeleteMapping("/gaf/users/{userId}")
    fun delete(@PathVariable userId: String) : ErrorResult {
        service.delete(userId)
        return ErrorResult(0,"deleted")
    }

    @PostMapping("/gaf/users/{userId}")
    fun update(@PathVariable userId: String, @Valid @RequestBody data: UpdateForm) : GafUserInfo {
        return service.update(userId, data)
    }

}