package gaf3.demo.services.crud.service

import gaf3.core.data.PagedData
import gaf3.core.exception.BusinessError
import gaf3.core.jpa.extension.findAll
import gaf3.core.util.DataBeanHelper
import gaf3.demo.services.crud.dao.GafUserInfoDAO
import gaf3.demo.services.crud.domain.CreateForm
import gaf3.demo.services.crud.domain.UpdateForm
import gaf3.demo.services.crud.entity.GafUserInfo
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.data.domain.PageRequest
import org.springframework.data.domain.Sort
import org.springframework.data.repository.findByIdOrNull
import org.springframework.stereotype.Service
import org.springframework.util.Assert
import org.springframework.validation.annotation.Validated
import java.util.*
import javax.transaction.Transactional

@Service("DemoUserService")
@Validated
class UserService(@Autowired private val userDao: GafUserInfoDAO) {
//    companion object {
//        val log: Logger = LoggerFactory.getLogger(UserService::class.java)
//    }

    @Transactional
    fun create(data: CreateForm): GafUserInfo {
        // TODO: 验证信息
        Assert.hasText(data.name, "用户名不能为空")

        // TODO: 设置默认属性
        val userId = UUID.randomUUID().toString()
        var user = GafUserInfo()
        DataBeanHelper.Bean2Obj(data, user)
        user.userId = userId
        user.status = "0"

        // TODO: 保存数据
        user = userDao.save(user)
        return fetch(user.userId!!)!!
    }

    fun delete(userId: String) {
        // 验证信息
        Assert.hasText(userId, "用户ID不能为空")
        val entity: GafUserInfo = userDao.findById(userId).orElseThrow {
            throw BusinessError(BusinessError.ERR_DATA_NOTEXIST, "数据不存在")
        }
        if(entity.origin == "default") {
            throw BusinessError(BusinessError.ERR_SERVICE_FAULT, "默认用户不能删除")
        }
        // 删除数据
        userDao.deleteById(userId)
    }

    fun update(userId: String, data: UpdateForm): GafUserInfo {
        // TODO: 验证信息
        Assert.hasText(userId, "用户ID不能为空")

        // TODO: 设置修改的属性
        val user = userDao.findByIdOrNull(userId) ?:
            throw BusinessError(BusinessError.ERR_DATA_NOTEXIST, "用户数据不存在")
        DataBeanHelper.Bean2Obj(data, user)

        userDao.save(user)
        return fetch(userId)!!
    }

    fun fetch(userId: String): GafUserInfo? {
        // TODO: 验证信息
        Assert.hasText(userId, "用户ID不能为空")

        return userDao.findByIdOrNull(userId) ?: return null
    }

    @JvmOverloads
    fun find(filter: GafUserInfo, skip: Int = 0, limit: Int = 100, exact: Boolean = true): PagedData<GafUserInfo> {
        val page:Int = skip/limit
        // val paged = userDao.findAll(example, PageRequest.of(page, limit, Sort.Direction.ASC, "createdAt"))
        val paged = userDao.findAll(filter, PageRequest.of(page, limit, Sort.Direction.ASC, "createdAt"), exact)

        return PagedData(paged.content, paged.totalElements.toInt())
    }
}