package gaf3.demo.services.nodb

import gaf3.core.cache.CacheService
import gaf3.core.cache.GafCacheConfigure
import gaf3.core.cache.MemCacheService
import gaf3.core.cache.RedisCacheService
import org.springframework.beans.factory.annotation.Qualifier
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty
import org.springframework.boot.context.properties.ConfigurationProperties
import org.springframework.context.annotation.Bean
import org.springframework.context.annotation.ComponentScan
import org.springframework.context.annotation.Configuration
import org.springframework.data.redis.core.RedisTemplate

@Configuration
@ComponentScan(basePackageClasses = [DemoVerifyConfiguration::class])
class DemoVerifyConfiguration {

    @Bean
    @ConfigurationProperties(prefix = "gaf.cache.verify")
    fun demoVerifyCacheConfigure(): GafCacheConfigure {
        return GafCacheConfigure();
    }

    @ConditionalOnProperty(prefix = "gaf.cache.verify", name = ["useRedis"], havingValue = "true", matchIfMissing = false)
    @Configuration
    class RedisCache {
        @Bean
        fun demoVerifyCacheService(template: RedisTemplate<String,String>, @Qualifier("demoVerifyCacheConfigure") config: GafCacheConfigure): CacheService {
            return RedisCacheService(template, config)
        }
    }

    @ConditionalOnProperty(prefix = "gaf.cache.verify", name = ["useRedis"], havingValue = "false", matchIfMissing = true)
    @Configuration
    class MemCache {
        @Bean
        fun demoVerifyCacheService(@Qualifier("demoVerifyCacheConfigure") config: GafCacheConfigure): CacheService {
            return MemCacheService(config)
        }
    }
}